@@ -12,13 +12,13 @@ import ssl
 from datetime import datetime, timedelta
 from json.decoder import JSONDecodeError
 
-import decorator
 import urllib3
+from urllib3.connection import match_hostname as urllib3_match_hostname
 from urllib3.util.ssl_ import ssl_wrap_socket as urllib3_ssl_wrap_socket
 from urllib3.util.ssl_ import wrap_socket as urllib3_wrap_socket
 
 from .compat import basestring, byte_type, decode_from_bytes, encode_to_bytes, text_type
-from .utils import SSL_PROTOCOL, MocketSocketCore, hexdump, hexload, wrap_ssl_socket
+from .utils import SSL_PROTOCOL, MocketSocketCore, get_mocketize, hexdump, hexload
 
 xxh32 = None
 try:
@@ -49,24 +49,41 @@ true_ssl_context = ssl.SSLContext
 true_inet_pton = socket.inet_pton
 true_urllib3_wrap_socket = urllib3_wrap_socket
 true_urllib3_ssl_wrap_socket = urllib3_ssl_wrap_socket
+true_urllib3_match_hostname = urllib3_match_hostname
 
 
 class SuperFakeSSLContext(object):
-    """ For Python 3.6 """
+    """For Python 3.6"""
 
     class FakeSetter(int):
         def __set__(self, *args):
             pass
 
     options = FakeSetter()
-    verify_mode = FakeSetter(ssl.CERT_OPTIONAL)
+    verify_mode = FakeSetter(ssl.CERT_NONE)
 
 
 class FakeSSLContext(SuperFakeSSLContext):
+    DUMMY_METHODS = (
+        "load_default_certs",
+        "load_verify_locations",
+        "set_alpn_protocols",
+    )
     sock = None
     post_handshake_auth = None
+    _check_hostname = False
+
+    @property
+    def check_hostname(self):
+        return self._check_hostname
+
+    @check_hostname.setter
+    def check_hostname(self, *args):
+        self._check_hostname = False
 
     def __init__(self, sock=None, server_hostname=None, _context=None, *args, **kwargs):
+        self._set_dummy_methods()
+
         if isinstance(sock, MocketSocket):
             self.sock = sock
             self.sock._host = server_hostname
@@ -78,13 +95,12 @@ class FakeSSLContext(SuperFakeSSLContext):
         elif isinstance(sock, int) and true_ssl_context:
             self.context = true_ssl_context(sock)
 
-    @staticmethod
-    def load_default_certs(*args, **kwargs):
-        pass
+    def _set_dummy_methods(self):
+        def dummy_method(*args, **kwargs):
+            pass
 
-    @staticmethod
-    def load_verify_locations(*args, **kwargs):
-        pass
+        for m in self.DUMMY_METHODS:
+            setattr(self, m, dummy_method)
 
     @staticmethod
     def wrap_socket(sock=sock, *args, **kwargs):
@@ -133,7 +149,6 @@ class MocketSocket(object):
         self, family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0, *args, **kwargs
     ):
         self.true_socket = true_socket(family, type, proto)
-        self._connected = False
         self._buflen = 65536
         self._entry = None
         self.family = int(family)
@@ -142,16 +157,6 @@ class MocketSocket(object):
         self._truesocket_recording_dir = None
         self.kwargs = kwargs
 
-        sock = kwargs.get("sock")
-        if sock is not None:
-            self.__dict__ = dict(sock.__dict__)
-
-            self.true_socket = wrap_ssl_socket(
-                true_ssl_socket,
-                self.true_socket,
-                true_ssl_context(protocol=SSL_PROTOCOL),
-            )
-
     def __unicode__(self):  # pragma: no cover
         return str(self)
 
@@ -160,6 +165,12 @@ class MocketSocket(object):
             self.__class__.__name__, self.family, self.type, self.proto
         )
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
     @property
     def fd(self):
         if self._fd is None:
@@ -318,16 +329,10 @@ class MocketSocket(object):
             host = true_gethostbyname(host)
 
             if isinstance(self.true_socket, true_socket) and self._secure_socket:
-                try:
-                    self = MocketSocket(sock=self)
-                except TypeError:
-                    ssl_context = self.kwargs.get("ssl_context")
-                    server_hostname = self.kwargs.get("server_hostname")
-                    self.true_socket = true_ssl_context.wrap_socket(
-                        self=ssl_context,
-                        sock=self.true_socket,
-                        server_hostname=server_hostname,
-                    )
+                self.true_socket = true_urllib3_ssl_wrap_socket(
+                    self.true_socket,
+                    **self.kwargs,
+                )
 
             try:
                 self.true_socket.connect((host, port))
@@ -368,7 +373,7 @@ class MocketSocket(object):
 
     def send(self, data, *args, **kwargs):  # pragma: no cover
         entry = self.get_entry(data)
-        if entry and self._entry != entry:
+        if not entry or (entry and self._entry != entry):
             self.sendall(data, entry=entry, *args, **kwargs)
         else:
             req = Mocket.last_request()
@@ -378,10 +383,12 @@ class MocketSocket(object):
         return len(data)
 
     def close(self):
+        if self.true_socket and not self.true_socket._closed:
+            self.true_socket.close()
         self._fd = None
 
     def __getattr__(self, name):
-        """ Do nothing catchall function, for methods like close() and shutdown() """
+        """Do nothing catchall function, for methods like close() and shutdown()"""
 
         def do_nothing(*args, **kwargs):
             pass
@@ -472,6 +479,9 @@ class Mocket(object):
         urllib3.connection.ssl_wrap_socket = urllib3.connection.__dict__[
             "ssl_wrap_socket"
         ] = FakeSSLContext.wrap_socket
+        urllib3.connection.match_hostname = urllib3.connection.__dict__[
+            "match_hostname"
+        ] = lambda cert, hostname: None
         if pyopenssl_override:  # pragma: no cover
             # Take out the pyopenssl version - use the default implementation
             extract_from_urllib3()
@@ -499,6 +509,9 @@ class Mocket(object):
         urllib3.connection.ssl_wrap_socket = urllib3.connection.__dict__[
             "ssl_wrap_socket"
         ] = true_urllib3_ssl_wrap_socket
+        urllib3.connection.match_hostname = urllib3.connection.__dict__[
+            "match_hostname"
+        ] = true_urllib3_match_hostname
         Mocket.reset()
         if pyopenssl_override:  # pragma: no cover
             # Put the pyopenssl version back in place
@@ -514,7 +527,7 @@ class Mocket(object):
 
     @classmethod
     def assert_fail_if_entries_not_served(cls):
-        """ Mocket checks that all entries have been served at least once. """
+        """Mocket checks that all entries have been served at least once."""
         assert all(
             entry._served for entry in itertools.chain(*cls._entries.values())
         ), "Some Mocket entries have not been served"
@@ -541,18 +554,18 @@ class MocketEntry(object):
         ):
             responses = [responses]
 
-        self.responses = []
-        for r in responses:
-            if isinstance(r, BaseException):
-                pass
-            elif not getattr(r, "data", False):
-                if isinstance(r, text_type):
-                    r = encode_to_bytes(r)
-                r = self.response_cls(r)
-            self.responses.append(r)
+        if not responses:
+            self.responses = [self.response_cls(encode_to_bytes(""))]
         else:
-            if not responses:
-                self.responses = [self.response_cls(encode_to_bytes(""))]
+            self.responses = []
+            for r in responses:
+                if isinstance(r, BaseException):
+                    pass
+                elif not getattr(r, "data", False):
+                    if isinstance(r, text_type):
+                        r = encode_to_bytes(r)
+                    r = self.response_cls(r)
+                self.responses.append(r)
 
     def can_handle(self, data):
         return True
@@ -580,47 +593,56 @@ class Mocketizer(object):
         self.truesocket_recording_dir = truesocket_recording_dir
         self.namespace = namespace or text_type(id(self))
 
-    def __enter__(self):
+    def enter(self):
         Mocket.enable(
             namespace=self.namespace,
             truesocket_recording_dir=self.truesocket_recording_dir,
         )
         if self.instance:
             self.check_and_call("mocketize_setup")
+
+    def __enter__(self):
+        self.enter()
         return self
 
-    def __exit__(self, type, value, tb):
+    def exit(self):
         if self.instance:
             self.check_and_call("mocketize_teardown")
         Mocket.disable()
 
+    def __exit__(self, type, value, tb):
+        self.exit()
+
+    async def __aenter__(self, *args, **kwargs):
+        self.enter()
+        return self
+
+    async def __aexit__(self, *args, **kwargs):
+        self.exit()
+
     def check_and_call(self, method):
         method = getattr(self.instance, method, None)
         if callable(method):
             method()
 
-    @classmethod
-    def wrap(cls, test=None, truesocket_recording_dir=None):
-        def wrapper(t, *args, **kw):
-            instance = args[0] if args else None
-            namespace = None
-            if truesocket_recording_dir:
-                namespace = ".".join(
-                    (
-                        instance.__class__.__module__,
-                        instance.__class__.__name__,
-                        t.__name__,
-                    )
-                )
-            with cls(
-                instance,
-                namespace=namespace,
-                truesocket_recording_dir=truesocket_recording_dir,
-            ):
-                t(*args, **kw)
-            return wrapper
 
-        return decorator.decorator(wrapper, test)
+def wrapper(test, cls=Mocketizer, truesocket_recording_dir=None, *args, **kwargs):
+    instance = args[0] if args else None
+    namespace = None
+    if truesocket_recording_dir:
+        namespace = ".".join(
+            (
+                instance.__class__.__module__,
+                instance.__class__.__name__,
+                test.__name__,
+            )
+        )
+    with cls(
+        instance,
+        namespace=namespace,
+        truesocket_recording_dir=truesocket_recording_dir,
+    ):
+        return test(*args, **kwargs)
 
 
-mocketize = Mocketizer.wrap
+mocketize = get_mocketize(wrapper_=wrapper)
