@@ -1,8 +1,10 @@
 from __future__ import unicode_literals
 
+import os
 import io
 import socket
 from unittest import TestCase
+from unittest.mock import patch
 
 import pytest
 
@@ -127,6 +129,16 @@ class MocketTestCase(TestCase):
             self.assertEqual(fp.read().strip(), encode_to_bytes("Show me."))
             self.assertEqual(len(Mocket._requests), 1)
 
+    def test_socket_as_context_manager(self):
+        addr = ("localhost", 80)
+        Mocket.register(MocketEntry(addr, ["Show me.\r\n"]))
+        with Mocketizer():
+            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as _so:
+                _so.connect(addr)
+                _so.sendall(encode_to_bytes("Whatever..."))
+                data = _so.recv(4096)
+                self.assertEqual(data, encode_to_bytes("Show me.\r\n"))
+
 
 class MocketizeTestCase(TestCase):
     def mocketize_setup(self):
@@ -153,3 +165,12 @@ def fixture():
 @mocketize
 def test_mocketize_with_fixture(fixture):
     assert 2 == fixture
+
+
+@mocketize
+@patch("os.getcwd")
+def test_patch(
+    method_patch,
+):
+    method_patch.return_value = 'foo'
+    assert os.getcwd() == "foo"
